package nl.amis.app.model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import oracle.jdbc.OracleStatement;
import oracle.jdbc.dcn.DatabaseChangeEvent;
import oracle.jdbc.dcn.DatabaseChangeListener;
import oracle.jdbc.dcn.DatabaseChangeRegistration;
import oracle.jdbc.driver.OracleConnection;

public class DatabaseNotificationHandler implements DatabaseChangeListener{

    private static Boolean registered = Boolean.FALSE;

private static List<DataUpdateEventListener> listeners = new ArrayList<DataUpdateEventListener>();
    private static final String REGISTRATION_SQL = "select id, event_context from db_notifications where event_type='EMP_DML6'";

    public static void addListener(DataUpdateEventListener listener) {
  listeners.add(listener);
}

    public static void registerForDatabaseNotifications(OracleConnection conn) {
        System.out.println("Register for table DB_NOTIFICATIONS "+REGISTRATION_SQL);
        if (!registered) {
           Properties prop = new Properties();
           prop.setProperty(OracleConnection.DCN_NOTIFY_ROWIDS,"true");
        //
        //Set the DCN_QUERY_CHANGE_NOTIFICATION option for query registration with finer granularity.
        prop.setProperty(OracleConnection.DCN_QUERY_CHANGE_NOTIFICATION,"true");
            prop.setProperty(OracleConnection.DCN_IGNORE_DELETEOP,"true");
            prop.setProperty(OracleConnection.DCN_IGNORE_UPDATEOP,"true");
            prop.setProperty(OracleConnection.DCN_BEST_EFFORT,"true");
        
           // The following operation does a roundtrip to the database to create a new
           // registration for DCN. It sends the client address (ip address and port) that
           // the server will use to connect to the client and send the notification
           // when necessary. Note that for now the registration is empty (we haven't registered
           // any table). This also opens a new thread in the drivers. This thread will be
           // dedicated to DCN (accept connection to the server and dispatch the events to 
           // the listeners).
           DatabaseChangeRegistration dcr=null;

        try {
            dcr = conn.registerDatabaseChangeNotification(prop);
        } catch (SQLException e) {
        }
        try
           {
             // add the listenerr:
            
             dcr.addListener(new DatabaseNotificationHandler());
              
             // second step: add objects in the registration:
             Statement stmt = conn.createStatement();
             // associate the statement with the registration:
             ((OracleStatement)stmt).setDatabaseChangeRegistration(dcr);
             ResultSet rs = stmt.executeQuery(REGISTRATION_SQL);
             while (rs.next())
             {}
             String[] tableNames = dcr.getTables();
             for(int i=0;i<tableNames.length;i++)
               System.out.println(tableNames[i]+" is part of the registration.");
             rs.close();
             stmt.close();
           }
           catch(SQLException ex)
           {
             // if an exception occurs, we need to close the registration in order
             // to interrupt the thread otherwise it will be hanging around.
             if(conn != null)
                try {
                    conn.unregisterDatabaseChangeNotification(dcr);
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            ex.printStackTrace();
           }
          
        registered = true;
        }
        else {
            System.out.println("--already registered before");
        }
        
    }

    public void onDatabaseChangeNotification(DatabaseChangeEvent databaseChangeEvent) {
        // at this point we can also requery the notifications table for all entries created since our previous query
        // and publish events for each row we find
        System.out.println("event received");
        int index=1;
        for (DataUpdateEventListener listener:listeners) {
            System.out.println("Inform listener "+index++);
            listener.processEvent("EMP", "no context available right now");
        }
    }
}
